"""
QKA策略模块

提供策略开发的抽象基类，定义策略开发的标准接口和事件处理机制。
"""

from abc import ABC, abstractmethod
from qka.core.broker import Broker

class Strategy(ABC):
    """
    策略抽象基类
    
    所有自定义策略都应该继承此类，并实现on_bar方法。
    
    Attributes:
        broker (Broker): 交易经纪商实例，用于执行交易操作
    """
    
    def __init__(self):
        """初始化策略"""
        self.broker = Broker()
    
    @abstractmethod
    def on_bar(self, date, get):
        """
        每个bar的处理逻辑，必须由子类实现
        
        Args:
            date: 当前时间戳
            get: 获取因子数据的函数，格式为 get(factor_name) -> pd.Series
        """
        pass


"""
QKA回测引擎模块

提供基于时间序列的回测功能，支持多股票横截面数据处理，包含完整的交易记录和可视化功能。
"""

import plotly.graph_objects as go

class Backtest:
    """
    QKA回测引擎类
    
    提供基于时间序列的回测功能，支持多股票横截面数据处理。
    
    Attributes:
        data (Data): 数据对象实例
        strategy (Strategy): 策略对象实例
    """
    
    def __init__(self, data, strategy):
        """
        初始化回测引擎
        
        Args:
            data (Data): Data类的实例，包含股票数据
            strategy (Strategy): 策略对象，必须包含on_bar方法
        """
        self.data = data
        self.strategy = strategy
    
    def run(self):
        """
        执行回测
        
        遍历所有时间点，在每个时间点调用策略的on_bar方法进行交易决策，
        并记录交易后的状态。
        """
        # 获取所有股票数据（dask DataFrame）
        df = self.data.get()

        for date, row in df.iterrows():
            def get(factor):
                """
                获取指定因子的数据
                
                Args:
                    factor (str): 因子名称，如 'close', 'volume' 等
                    
                Returns:
                    pd.Series: 该因子在所有股票上的值
                """
                s = row[row.index.str.endswith(factor)]
                s.index = s.index.str.replace(f'_{factor}$', '', regex=True)
                return s
            
            # 先调用策略的on_bar（可能包含交易操作）
            self.strategy.on_bar(date, get)
            
            # 再调用broker的on_bar记录状态（包含交易后的状态）
            self.strategy.broker.on_bar(date, get)
    
    def plot(self, title="回测收益曲线"):
        """
        绘制回测收益曲线图
        
        Args:
            title (str): 图表标题，默认为"回测收益曲线"
            
        Returns:
            plotly.graph_objects.Figure: 交互式图表对象，如果无数据则返回None
        """
        
        # 获取交易数据
        trades_df = self.strategy.broker.trades
        
        # 检查是否有数据
        if trades_df.empty or 'total' not in trades_df.columns:
            print("错误：没有可用的回测数据或缺少total列")
            return None
        
        # 提取总资产数据
        total_assets = trades_df['total']
        
        # 创建交互式图表
        fig = go.Figure()
        
        fig.add_trace(go.Scatter(
            x=total_assets.index,
            y=total_assets.values,
            mode='lines',
            name='总资产',
            line=dict(color='#2E86AB', width=3),
            hovertemplate='日期: %{x}<br>总资产: ¥%{y:,.2f}<extra></extra>'
        ))
        
        # 更新布局
        fig.update_layout(
            title=dict(
                text=title,
                x=0.5,
                font=dict(size=16)
            ),
            xaxis_title="日期",
            yaxis_title="总资产 (¥)",
            height=600,
            showlegend=True,
            template='plotly_white',
            hovermode='x unified'
        )
        
        # 添加网格线
        fig.update_xaxes(showgrid=True, gridwidth=1, gridcolor='LightGrey')
        
        # 显示图表
        fig.show()
        
        return fig


"""
QKA Core 模块
包含核心功能：数据处理、回测引擎、配置管理、事件系统等
"""


# 数据相关
from .data import Data
# 回测相关
from .backtest import Backtest
from .strategy import Strategy
from .broker import Broker

__all__ = [
    # 数据相关
    'Data',
    # 回测相关
    'Backtest', 'Strategy', 'Broker',
]



"""
QKA Utils 模块
包含通用工具函数和类
"""

from .logger import logger, create_logger, get_structured_logger
from .anis import RED, GREEN, YELLOW, BLUE, RESET
from .util import timestamp_to_datetime_string, parse_order_type, convert_to_current_date

__all__ = [
    'logger', 'create_logger', 'get_structured_logger',
    'RED', 'GREEN', 'YELLOW', 'BLUE', 'RESET',
    'timestamp_to_datetime_string', 'parse_order_type', 'convert_to_current_date'
]



from typing import Dict, Any, Optional, Callable
import asyncio
from mcp.server import FastMCP
import pandas as pd
import os
from datetime import datetime

class ModelServer:
    """模型服务器"""
    
    def __init__(self, port: int = 8080):
        """
        初始化模型服务器
        
        Args:
            port: 服务器端口
        """
        self.port = port
        self.models = {}
        self.running = False
    
    def register_model(self, name: str, model_func: Callable):
        """
        注册模型
        
        Args:
            name: 模型名称
            model_func: 模型函数
        """
        self.models[name] = model_func
    
    async def call_model(self, model_name: str, **kwargs) -> Any:
        """
        调用模型
        
        Args:
            model_name: 模型名称
            **kwargs: 模型参数
            
        Returns:
            模型输出结果
        """
        if model_name not in self.models:
            raise ValueError(f"模型 {model_name} 不存在")
        
        model_func = self.models[model_name]
        return await model_func(**kwargs)
    
    async def start(self):
        """启动模型服务器"""
        self.running = True
        print(f"模型服务器启动在端口 {self.port}")
    
    async def stop(self):
        """停止模型服务器"""
        self.running = False
        print("模型服务器已停止")


app = FastMCP('qka')

@app.tool()
def query_akshare_data(
        code: str = ''
) -> dict:
    """
    查询akshare数据源数据
    :param code: 查询代码，要求使用akshare库, 要求生成的函数名叫query,返回数据格式为pandas.DataFrame
    :return: 查询结果
    """
    # 创建本地命名空间，预先导入akshare
    local_namespace = {}
    global_namespace = globals().copy()
    
    try:
        # 尝试导入akshare到全局命名空间
        import akshare as ak
        global_namespace['akshare'] = ak
        global_namespace['ak'] = ak
    except ImportError:
        return {"error": "无法导入akshare模块，请确认已正确安装"}
    
    # 执行传入的代码
    try:
        exec(code, global_namespace, local_namespace)
        
        # 检查是否定义了query函数
        if 'query' not in local_namespace or not callable(local_namespace['query']):
            return {"error": "代码中未定义query函数"}
        
        # 执行query函数（不传入参数）
        result = local_namespace['query']()
        
        # 检查返回结果是否为DataFrame
        if not isinstance(result, pd.DataFrame):
            return {"error": "query函数返回的结果不是DataFrame类型"}
        
        # 创建保存目录
        save_dir = r"e:\Code\qka\output"
        os.makedirs(save_dir, exist_ok=True)
        
        # 生成文件名
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        filename = f"data_{timestamp}.csv"
        file_path = os.path.join(save_dir, filename)
        
        # 保存为CSV文件
        result.to_csv(file_path, index=False, encoding='utf-8-sig')
        
        # 将DataFrame转换为字典返回，并包含条数信息和文件路径
        data_dict = result.to_dict(orient="records")
        
        return {
            "message": "数据已成功保存为CSV文件",
            "file_path": file_path,
            "record_count": len(data_dict),
            "data": data_dict[:10] if len(data_dict) > 10 else data_dict  # 只返回前10条预览
        }
    
    except Exception as e:
        return {"error": f"报错后不需要重试，执行代码时发生错误: {str(e)}"}


if __name__ == '__main__':
    app.run(transport='stdio')


"""
QKA - 量化交易框架

统一的访问接口，支持 qka.xxx 的访问模式
"""

from importlib.metadata import version, PackageNotFoundError

try:
    __version__ = version("qka")
except PackageNotFoundError:
    __version__ = "0.1.0"  # fallback version

# 核心功能直接导入
from qka.core.data import Data
from qka.core.backtest import Backtest
from qka.core.strategy import Strategy

# 子模块导入
from qka import core, utils, mcp

# 交易相关（有依赖的模块暂时不导入，避免导入错误）
# from qka.brokers.trade import create_trader
# from qka.brokers.client import QMTClient
# from qka.brokers.server import QMTServer

__all__ = [
    # 核心功能
    'Data', 'Backtest', 'Strategy',
    # 子模块
    'core', 'utils', 'mcp'
]


import logging
import requests
import os
import re
import json
from datetime import date, datetime
from logging.handlers import RotatingFileHandler
from pathlib import Path
from typing import Optional, Dict, Any
import threading

# 自定义格式化器
class JSONFormatter(logging.Formatter):
    """JSON格式的日志格式化器"""
    
    def format(self, record):
        log_entry = {
            'timestamp': datetime.fromtimestamp(record.created).isoformat(),
            'level': record.levelname,
            'logger': record.name,
            'message': record.getMessage(),
            'module': record.module,
            'function': record.funcName,
            'line': record.lineno
        }
        
        # 添加异常信息
        if record.exc_info:
            log_entry['exception'] = self.formatException(record.exc_info)
        
        # 添加自定义字段
        if hasattr(record, 'extra_fields'):
            log_entry.update(record.extra_fields)
            
        return json.dumps(log_entry, ensure_ascii=False)


class ColoredFormatter(logging.Formatter):
    """带颜色的控制台格式化器"""
    
    # ANSI颜色代码
    COLORS = {
        'DEBUG': '\033[36m',     # 青色
        'INFO': '\033[32m',      # 绿色
        'WARNING': '\033[33m',   # 黄色
        'ERROR': '\033[31m',     # 红色
        'CRITICAL': '\033[35m',  # 紫色
        'RESET': '\033[0m'       # 重置
    }
    
    def format(self, record):
        color = self.COLORS.get(record.levelname, self.COLORS['RESET'])
        reset = self.COLORS['RESET']
        
        # 格式化消息
        formatted = super().format(record)
        return f"{color}{formatted}{reset}"


class RemoveAnsiEscapeCodes(logging.Filter):
    """移除ANSI转义码的过滤器"""
    
    def filter(self, record):
        record.msg = re.sub(r'\033\[[0-9;]*m', '', str(record.msg))
        return True


class StructuredLogger:
    """结构化日志记录器"""
    
    def __init__(self, name: str = 'qka'):
        self.logger = logging.getLogger(name)
        self._lock = threading.Lock()
    
    def log(self, level: int, message: str, **kwargs):
        """记录结构化日志"""
        with self._lock:
            # 创建日志记录，附加额外字段
            if kwargs:
                record = self.logger.makeRecord(
                    self.logger.name, level, '', 0, message, (), None
                )
                record.extra_fields = kwargs
                self.logger.handle(record)
            else:
                self.logger.log(level, message)
    
    def debug(self, message: str, **kwargs):
        self.log(logging.DEBUG, message, **kwargs)
    
    def info(self, message: str, **kwargs):
        self.log(logging.INFO, message, **kwargs)
    
    def warning(self, message: str, **kwargs):
        self.log(logging.WARNING, message, **kwargs)
    
    def error(self, message: str, **kwargs):
        self.log(logging.ERROR, message, **kwargs)
    
    def critical(self, message: str, **kwargs):
        self.log(logging.CRITICAL, message, **kwargs)


def create_logger(
    name: str = 'qka',
    level: str = 'INFO',
    console_output: bool = True,
    file_output: bool = True,
    log_dir: str = 'logs',
    max_file_size: str = '10MB',
    backup_count: int = 10,
    json_format: bool = False,
    colored_console: bool = True
):
    """
    创建增强的日志记录器
    
    Args:
        name: 日志记录器名称
        level: 日志级别
        console_output: 是否输出到控制台
        file_output: 是否输出到文件
        log_dir: 日志文件目录
        max_file_size: 最大文件大小
        backup_count: 备份文件数量
        json_format: 是否使用JSON格式
        colored_console: 控制台是否使用颜色
    """
    logger = logging.getLogger(name)
    logger.setLevel(getattr(logging, level.upper()))
    
    # 清除现有处理器
    logger.handlers.clear()
    
    # 创建格式化器
    if json_format:
        file_formatter = JSONFormatter()
        console_formatter = JSONFormatter()
    else:
        file_formatter = logging.Formatter(
            '[%(levelname)s][%(asctime)s][%(name)s] %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        console_formatter = ColoredFormatter(
            '[%(levelname)s][%(asctime)s] %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        ) if colored_console else file_formatter

    # 控制台处理器
    if console_output:
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(console_formatter)
        logger.addHandler(stream_handler)

    # 文件处理器
    if file_output:
        # 创建日志文件夹
        Path(log_dir).mkdir(parents=True, exist_ok=True)
        
        # 解析文件大小
        size_map = {'KB': 1024, 'MB': 1024*1024, 'GB': 1024*1024*1024}
        size_unit = max_file_size[-2:].upper()
        size_value = int(max_file_size[:-2])
        max_bytes = size_value * size_map.get(size_unit, 1024*1024)
        
        # 创建轮转文件处理器
        file_handler = RotatingFileHandler(
            f"{log_dir}/{date.today().strftime('%Y-%m-%d')}.log",
            maxBytes=max_bytes,
            backupCount=backup_count,
            encoding='utf-8'
        )
        file_handler.setLevel(logging.DEBUG)
        file_handler.setFormatter(file_formatter)
        file_handler.addFilter(RemoveAnsiEscapeCodes())
        logger.addHandler(file_handler)

    return logger


def setup_logging_from_config(config_dict: Dict[str, Any]) -> logging.Logger:
    """从配置字典设置日志"""
    return create_logger(**config_dict)


def get_structured_logger(name: str = 'qka') -> StructuredLogger:
    """获取结构化日志记录器"""
    return StructuredLogger(name)


class WeChatHandler(logging.Handler):
    def __init__(self, webhook_url):
        super().__init__()
        self.webhook_url = webhook_url

    def emit(self, record):
        log_entry = self.format(record)
        payload = {
            "msgtype": "text",
            "text": {
                "content": log_entry
            }
        }
        try:
            response = requests.post(self.webhook_url, json=payload, timeout=5)
            response.raise_for_status()
        except Exception as e:
            print(f"发送微信消息失败: {e}")


def add_wechat_handler(logger_instance: logging.Logger, webhook_url: str, level: str = 'ERROR'):
    """
    为日志记录器添加微信通知处理器
    
    Args:
        logger_instance: 日志记录器实例
        webhook_url: 企业微信机器人webhook地址
        level: 通知级别
    """
    wechat_handler = WeChatHandler(webhook_url)
    wechat_handler.setLevel(getattr(logging, level.upper()))
    
    # 创建简单格式化器用于微信消息
    formatter = logging.Formatter(
        '[%(levelname)s][%(asctime)s] %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )
    wechat_handler.setFormatter(formatter)
    wechat_handler.addFilter(RemoveAnsiEscapeCodes())
    logger_instance.addHandler(wechat_handler)


# 创建默认日志记录器
logger = create_logger()


if __name__ == '__main__':
    # 测试日志系统
    test_logger = create_logger('test', colored_console=True)
    
    test_logger.debug("这是调试信息")
    test_logger.info("这是普通信息")
    test_logger.warning("这是警告信息")
    test_logger.error("这是错误信息")
    test_logger.critical("这是严重错误")
    
    # 测试结构化日志
    struct_logger = get_structured_logger('test_struct')
    struct_logger.info("结构化日志测试", 
                      user_id=123, 
                      action="login", 
                      ip="192.168.1.1")
    
    print("日志测试完成，请查看logs目录")



"""
QKA交易服务器模块

提供基于FastAPI的QMT交易服务器，将QMT交易接口封装为RESTful API。
"""

from fastapi import FastAPI, HTTPException, Header, Depends
from pydantic import BaseModel
import inspect
from typing import Any, Dict, Optional
from qka.brokers.trade import create_trader
import uvicorn
import uuid
import hashlib

class QMTServer:
    """
    QMT交易服务器类
    
    将QMT交易接口封装为RESTful API，支持远程调用交易功能。
    
    Attributes:
        account_id (str): 账户ID
        mini_qmt_path (str): miniQMT安装路径
        host (str): 服务器地址
        port (int): 服务器端口
        app (FastAPI): FastAPI应用实例
        trader: QMT交易对象
        account: 账户对象
        token (str): 访问令牌
    """
    
    def __init__(self, account_id: str, mini_qmt_path: str, host: str = "0.0.0.0", port: int = 8000, token: Optional[str] = None):
        """
        初始化交易服务器
        
        Args:
            account_id (str): 账户ID
            mini_qmt_path (str): miniQMT路径
            host (str): 服务器地址，默认0.0.0.0
            port (int): 服务器端口，默认8000
            token (Optional[str]): 可选的自定义token
        """
        self.account_id = account_id
        self.mini_qmt_path = mini_qmt_path
        self.host = host
        self.port = port
        self.app = FastAPI()
        self.trader = None
        self.account = None
        self.token = token if token else self.generate_token()  # 使用自定义token或生成固定token
        print(f"\n授权Token: {self.token}\n")  # 打印token供客户端使用

    def generate_token(self) -> str:
        """
        生成基于机器码的固定token
        
        Returns:
            str: 生成的token字符串
        """
        # 获取机器码（例如MAC地址）
        mac = uuid.getnode()
        # 将机器码转换为字符串
        mac_str = str(mac)
        # 使用SHA256哈希生成固定长度的token
        token = hashlib.sha256(mac_str.encode()).hexdigest()
        return token

    async def verify_token(self, x_token: str = Header(...)):
        """
        验证token的依赖函数
        
        Args:
            x_token (str): 请求头中的token
            
        Returns:
            str: 验证通过的token
            
        Raises:
            HTTPException: token无效时抛出401错误
        """
        if x_token != self.token:
            raise HTTPException(status_code=401, detail="无效的Token")
        return x_token

    def init_trader(self):
        """初始化交易对象"""
        self.trader, self.account = create_trader(self.account_id, self.mini_qmt_path)

    def convert_to_dict(self, obj) -> Dict[str, Any]:
        """
        将结果转换为可序列化的字典
        
        Args:
            obj: 任意对象
            
        Returns:
            Dict[str, Any]: 可序列化的字典
        """
        # 如果是基本类型，直接返回
        if isinstance(obj, (int, float, str, bool)):
            return obj
        # 如果已经是字典类型，直接返回
        elif isinstance(obj, dict):
            return obj
        # 如果是列表或元组，递归转换每个元素
        elif isinstance(obj, (list, tuple)):
            return [self.convert_to_dict(item) for item in obj]
        # 如果是自定义对象，获取所有公开属性
        elif hasattr(obj, '__dir__'):
            attrs = obj.__dir__()
            # 过滤掉内部属性和方法
            public_attrs = {attr: getattr(obj, attr)
                           for attr in attrs
                           if not attr.startswith('_') and not callable(getattr(obj, attr))}
            return public_attrs
        # 其他类型直接返回
        return str(obj)  # 将无法处理的类型转换为字符串

    def convert_method_to_endpoint(self, method_name: str, method):
        """
        将 XtQuantTrader 方法转换为 FastAPI 端点
        
        Args:
            method_name (str): 方法名称
            method: 方法对象
        """
        sig = inspect.signature(method)
        param_names = list(sig.parameters.keys())
        
        # 创建动态的请求模型
        class_fields = {
            '__annotations__': {}  # 添加类型注解字典
        }
        
        for param in param_names:
            if param in ['self', 'account']:
                continue
            class_fields['__annotations__'][param] = Any
            class_fields[param] = None

        RequestModel = type(f'{method_name}Request', (BaseModel,), class_fields)

        async def endpoint(request: RequestModel, token: str = Depends(self.verify_token)):
            try:
                params = request.dict(exclude_unset=True)
                if 'account' in param_names:
                    params['account'] = self.account
                result = getattr(self.trader, method_name)(**params)
                converted_result = self.convert_to_dict(result)
                return {'success': True, 'data': converted_result}
            except Exception as e:
                raise HTTPException(status_code=500, detail=str(e))

        self.app.post(f'/api/{method_name}')(endpoint)

    def setup_routes(self):
        """设置所有路由"""
        trader_methods = inspect.getmembers(
            self.trader.__class__,
            predicate=lambda x: inspect.isfunction(x) or inspect.ismethod(x)
        )
        
        excluded_methods = {'__init__', '__del__', 'register_callback', 'start', 'stop',
                          'connect', 'sleep', 'run_forever', 'set_relaxed_response_order_enabled'}
        
        for method_name, method in trader_methods:
            if not method_name.startswith('_') and method_name not in excluded_methods:
                self.convert_method_to_endpoint(method_name, method)

    def start(self):
        """启动服务器"""
        self.init_trader()
        self.setup_routes()
        uvicorn.run(self.app, host=self.host, port=self.port)

def qmt_server(account_id: str, mini_qmt_path: str, host: str = "0.0.0.0", port: int = 8000, token: Optional[str] = None):
    """
    快速创建并启动服务器的便捷函数
    
    Args:
        account_id (str): 账户ID
        mini_qmt_path (str): miniQMT路径
        host (str): 服务器地址，默认0.0.0.0
        port (int): 服务器端口，默认8000
        token (Optional[str]): 可选的自定义token
    """
    server = QMTServer(account_id, mini_qmt_path, host, port, token)
    server.start()


"""
QKA数据模块

提供统一的数据获取、缓存和管理功能，支持多数据源、多周期、多因子的数据获取。
"""

from pathlib import Path
from tqdm import tqdm
from concurrent.futures import ThreadPoolExecutor, as_completed
import pandas as pd
import pyarrow as pa
import pyarrow.parquet as pq
import akshare as ak
import dask.dataframe as dd
from typing import List, Dict, Optional, Callable
from qka.utils.logger import logger

class Data():
    """
    数据管理类
    
    负责股票数据的获取、缓存和管理，支持多数据源、并发下载和自定义因子计算。
    
    Attributes:
        symbols (List[str]): 股票代码列表
        period (str): 数据周期，如 '1d'、'1m' 等
        adjust (str): 复权方式，如 'qfq'、'hfq'、'bfq'
        factor (Callable): 因子计算函数
        source (str): 数据源，如 'akshare'、'qmt'
        pool_size (int): 并发下载线程数
        datadir (Path): 数据缓存目录
        target_dir (Path): 目标存储目录
    """
    
    def __init__(
        self, 
        symbols: Optional[List[str]] = None,
        period: str = '1d',
        adjust: str = 'qfq',
        factor: Callable[[pd.DataFrame], pd.DataFrame] = lambda df: df,
        source: str = 'akshare',
        pool_size: int = 10,
        datadir: Optional[Path] = None
    ):
        """
        初始化数据对象
        
        Args:
            symbols: [维度1] 标的，如 ['000001.SZ', '600000.SH']
            period: [维度2] 周期，如 '1m', '5m', '1d' 等
            factor: [维度3] 因子字典，key为因子名，value为因子函数
            source: [维度4] 数据来源 ('qmt', 'akshare')
            pool_size: 并发池大小
            datadir: 缓存根目录，默认为项目根目录下的 datadir
        """
        self.symbols = symbols or []
        self.period = period
        self.adjust = adjust
        self.factor = factor
        self.source = source
        self.pool_size = pool_size

        # 初始化缓存目录
        if datadir is None:
            # 默认使用当前工作目录下的 datadir
            self.datadir = Path.cwd() / "datadir"
        else:
            self.datadir = Path(datadir)
        
        self.datadir.mkdir(parents=True, exist_ok=True)

        self.target_dir = self.datadir / self.source / self.period / (self.adjust or "bfq")
        self.target_dir.mkdir(parents=True, exist_ok=True)

    def _download(self, symbol: str) -> Path:
        """
        下载单个股票的数据
        
        Args:
            symbol (str): 股票代码
            
        Returns:
            Path: 数据文件路径
        """
        path = self.target_dir / f"{symbol}.parquet"

        if path.exists():
             return path

        if self.source == 'akshare':
            df = self._get_from_akshare(symbol)
        else:
            df = pd.DataFrame()

        if len(df) == 0:
            return path

        table = pa.Table.from_pandas(df)
        pq.write_table(table, path)

        return path

    def get(self) -> dd.DataFrame:
        """
        获取历史数据
        
        并发下载所有股票数据，应用因子计算，并返回合并后的Dask DataFrame。
        
        Returns:
            dd.DataFrame: 合并后的股票数据，每只股票的列名格式为 {symbol}_{column}
        """
        # 准备缓存目录

        with ThreadPoolExecutor(max_workers=self.pool_size) as executor:
            # 提交下载任务
            futures = {
                executor.submit(self._download, symbol): symbol
                for symbol in self.symbols
            }

            # 添加tqdm进度条
            with tqdm(total=len(self.symbols), desc="下载数据") as pbar:
                for future in as_completed(futures):
                    symbol = futures[future]
                    pbar.update(1)
                    pbar.set_postfix_str(f"当前: {symbol}")

        dfs = []
        for symbol in self.symbols:
            df = dd.read_parquet(str(self.target_dir / f"{symbol}.parquet"))
            df = self.factor(df)
            column_mapping = {col: f'{symbol}_{col}' for col in df.columns}
            dfs.append(df.rename(columns=column_mapping))

        df = dd.concat(dfs, axis=1, join='outer')

        return df

    def _get_from_akshare(self, symbol: str) -> pd.DataFrame:
        """
        从 akshare 获取单个股票的数据。

        Args:
            symbol (str): 股票代码，支持带后缀如 000001.SZ 或不带后缀的 000001

        Returns:
            pd.DataFrame: 股票数据，以 date 为索引，包含 open, high, low, close, volume, amount 列
        """
        column_mapping = {
            "日期": "date",
            "开盘": "open",
            "收盘": "close",
            "最高": "high",
            "最低": "low",
            "成交量": "volume",
            "成交额": "amount",
        }

        # 下载数据
        df = ak.stock_zh_a_hist(symbol=symbol, period='daily', adjust=self.adjust)

        # 数据标准化处理
        # 1. 标准化列名
        df = df.rename(columns=column_mapping)
        if "date" in df.columns:
            df["date"] = pd.to_datetime(df["date"])

        # 2. 确保数值列为数值类型
        numeric_cols = [c for c in ("open", "high", "low", "close", "volume", "amount") if c in df.columns]
        for col in numeric_cols:
            df[col] = pd.to_numeric(df[col], errors="coerce")

        # 3. 只保留需要的列
        mapped_columns = list(column_mapping.values())
        available_columns = [col for col in mapped_columns if col in df.columns]
        df = df[available_columns]

        df = df.set_index('date')
        # 设置索引
        return df



"""
QKA经纪商模块

提供虚拟交易经纪商功能，管理资金、持仓和交易记录，支持回测环境下的交易操作。
"""

import pandas as pd
from typing import Dict, List, Any, Optional

class Broker:
    """
    虚拟交易经纪商类
    
    管理资金、持仓和交易记录，提供买入卖出操作接口。
    
    Attributes:
        cash (float): 可用现金
        positions (Dict): 持仓记录，格式: {symbol: {'size': 数量, 'avg_price': 平均成本}}
        trade_history (List): 交易历史记录
        timestamp: 当前时间戳
        trades (pd.DataFrame): 交易记录DataFrame
    """
    
    def __init__(self, initial_cash=100000.0):
        """
        初始化Broker类
        
        Args:
            initial_cash (float): 初始资金，默认10万元
        """
        self.cash = initial_cash  # 可用现金
        self.positions = {}       # 持仓记录，格式: {symbol: {'size': 数量, 'avg_price': 平均成本}}
        self.trade_history = []   # 交易历史记录
        self.timestamp = None     # 当前时间戳
        
        # 交易记录
        self.trades = pd.DataFrame(columns=[
            'cash', 'value', 'total',
            'positions', 'trades'
        ])
        
    def on_bar(self, date, get):
        """
        处理每个bar的数据，记录状态
        
        Args:
            date: 当前时间戳
            get: 获取因子数据的函数，格式为 get(factor_name) -> pd.Series
        """
        self.timestamp = date
        
        # 获取当前市场价格
        close_prices = get('close')
        market_prices = close_prices.to_dict() if hasattr(close_prices, 'to_dict') else {}
        
        # 记录状态到trades DataFrame
        if self.timestamp is None:
            return
            
        # 获取当日交易记录
        daily_trades = []
        for trade in self.trade_history:
            if trade['timestamp'] == self.timestamp:
                daily_trades.append(trade)
        
        # 计算持仓市值
        position_value = 0.0
        for symbol, position in self.positions.items():
            if market_prices and symbol in market_prices:
                # 使用市场价格计算市值
                position_value += position['size'] * market_prices[symbol]
            else:
                # 使用平均成本估算市值
                position_value += position['size'] * position['avg_price']
        
        # 计算总资产
        total_assets = self.cash + position_value
        
        # 记录状态
        state_data = {
            'cash': self.cash,
            'value': position_value,
            'total': total_assets,
            'positions': self.positions.copy(),
            'trades': daily_trades.copy()
        }
        
        # 添加到trades
        self.trades.loc[self.timestamp] = state_data
    
    def buy(self, symbol: str, price: float, size: int) -> bool:
        """
        买入操作
        
        Args:
            symbol (str): 交易标的代码
            price (float): 买入价格
            size (int): 买入数量
            
        Returns:
            bool: 交易是否成功
        """
        # 计算买入所需金额
        required_cash = price * size
        
        # 检查资金是否足够
        if self.cash < required_cash:
            print(f"资金不足！需要 {required_cash:.2f}，当前可用 {self.cash:.2f}")
            return False
        
        # 执行买入操作
        self.cash -= required_cash
        
        # 更新持仓
        if symbol in self.positions:
            # 已有持仓，计算新的平均成本
            old_position = self.positions[symbol]
            old_size = old_position['size']
            old_avg_price = old_position['avg_price']
            old_total_value = old_size * old_avg_price
            new_total_value = old_total_value + required_cash
            new_size = old_size + size
            new_avg_price = new_total_value / new_size
            
            self.positions[symbol] = {
                'size': new_size,
                'avg_price': new_avg_price
            }
        else:
            # 新建持仓
            self.positions[symbol] = {
                'size': size,
                'avg_price': price
            }
        
        # 记录交易历史
        self.trade_history.append({
            'action': 'buy',
            'symbol': symbol,
            'price': price,
            'size': size,
            'timestamp': self.timestamp
        })
        
        print(f"买入成功: {symbol} {size}股 @ {price:.2f}，花费 {required_cash:.2f}")
        return True
    
    def sell(self, symbol: str, price: float, size: int) -> bool:
        """
        卖出操作
        
        Args:
            symbol (str): 交易标的代码
            price (float): 卖出价格
            size (int): 卖出数量
            
        Returns:
            bool: 交易是否成功
        """
        # 检查是否有足够的持仓
        if symbol not in self.positions:
            print(f"没有 {symbol} 的持仓！")
            return False
        
        position = self.positions[symbol]
        if position['size'] < size:
            print(f"持仓不足！当前持有 {position['size']}，尝试卖出 {size}")
            return False
        
        # 计算卖出所得金额
        sale_proceeds = price * size
        
        # 执行卖出操作
        self.cash += sale_proceeds
        
        # 更新持仓
        if position['size'] == size:
            # 全部卖出，删除持仓记录
            del self.positions[symbol]
        else:
            # 部分卖出，更新持仓数量
            self.positions[symbol]['size'] -= size
        
        # 记录交易历史
        self.trade_history.append({
            'action': 'sell',
            'symbol': symbol,
            'price': price,
            'size': size,
            'timestamp': self.timestamp
        })
        
        print(f"卖出成功: {symbol} {size}股 @ {price:.2f}，获得 {sale_proceeds:.2f}")
        return True
    
    def get(self, factor: str, timestamp=None) -> Any:
        """
        从trades DataFrame中获取数据
        
        Args:
            factor (str): 列名，可选 'cash', 'value', 'total', 'positions', 'trades'
            timestamp: 时间戳，如果为None则使用当前时间戳
            
        Returns:
            Any: 对应列的数据，如果不存在则返回None
        """
        if timestamp is None:
            timestamp = self.timestamp
        
        if timestamp is None or timestamp not in self.trades.index:
            return None
            
        return self.trades.at[timestamp, factor]
    
    



"""
QKA交易客户端模块

提供QMT交易服务器的客户端接口，支持远程调用交易功能。
"""

import requests
from typing import Any, Dict, Optional
from qka.utils.logger import logger

class QMTClient:
    """
    QMT交易客户端类
    
    提供与QMT交易服务器的通信接口，支持各种交易操作。
    
    Attributes:
        base_url (str): API服务器地址
        session (requests.Session): HTTP会话对象
        token (str): 访问令牌
        headers (Dict): HTTP请求头
    """
    
    def __init__(self, base_url: str = "http://localhost:8000", token: Optional[str] = None):
        """
        初始化交易客户端
        
        Args:
            base_url (str): API服务器地址，默认为本地8000端口
            token (str): 访问令牌，必须与服务器的token一致
            
        Raises:
            ValueError: 如果未提供token
        """
        self.base_url = base_url.rstrip('/')
        self.session = requests.Session()
        if not token:
            raise ValueError("必须提供访问令牌(token)")
        self.token = token
        self.headers = {"X-Token": self.token}

    def api(self, method_name: str, **params) -> Any:
        """
        通用调用接口方法
        
        Args:
            method_name (str): 要调用的接口名称
            **params: 接口参数，作为关键字参数传入
            
        Returns:
            Any: 接口返回的数据
            
        Raises:
            Exception: API调用失败时抛出异常
        """
        try:
            response = self.session.post(
                f"{self.base_url}/api/{method_name}",
                json=params or {},
                headers=self.headers
            )
            response.raise_for_status()
            result = response.json()
            
            if not result.get('success'):
                raise Exception(f"API调用失败: {result.get('detail')}")
            
            return result.get('data')
        except Exception as e:
            logger.error(f"调用 {method_name} 失败: {str(e)}")
            raise

